/*
 * Copyright 2011 BitMeister Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.bitmeister.asn1.codec.ber;

import java.io.IOException;
import java.io.OutputStream;

/**
 * Abstract base class for classes represent BER encoded octets.
 * 
 * @author WATANABE, Jun. <jwat at bitmeister.jp>
 */
abstract class BerOctets {

	private byte[] identifier = new byte[0];

	private byte[] length = new byte[0];

	/**
	 * Returns BER encoded octets length includes prefix and contents.
	 * 
	 * @return Total length of BER encoded octets.
	 */
	int totalLength() {
		return identifier.length + length.length + octetsLength();
	}

	/**
	 * Sets identifier and length octets.
	 * 
	 * @param identifier
	 *            BER encoded identifier octets.
	 * @param length
	 *            BER encoded contents length.
	 */
	void setPrefix(byte[] identifier, byte[] length) {
		this.identifier = identifier;
		this.length = length;
	}

	/**
	 * Writes all BER encoded octets to the {@code OutputStream}
	 * 
	 * @param out
	 *            The stream to be written.
	 * @throws IOException
	 *             when {@code IOException} thrown by {@code OutputStream}.
	 */
	void write(OutputStream out) throws IOException {
		out.write(identifier);
		out.write(length);
		writeContents(out);
	}

	/**
	 * Returns this instance is constructed or not.
	 * 
	 * @return {@code true} if this instance is constructed.
	 */
	abstract boolean isConstructed();

	/**
	 * Returns length of contents octets.
	 * 
	 * @return contents octets length.
	 */
	abstract int octetsLength();

	/**
	 * Writes contents octets to the {@code OutputStream}.
	 * 
	 * @param out
	 *            The stream to be written.
	 * @throws IOException
	 *             when {@code OutputStream} throws {@code IOException}.
	 */
	abstract void writeContents(OutputStream out) throws IOException;

}
