/*
 * Copyright 2011 BitMeister Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jp.bitmeister.asn1.codec.ber;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents constructed BER octets.
 * 
 * <p>
 * This class represents a BER encoded octets that contains some contents
 * octets.
 * </p>
 * 
 * @author WATANABE, Jun. <jwat at bitmeister.jp>
 */
class ConstructedOctets extends BerOctets {

	private List<BerOctets> elements = new ArrayList<BerOctets>();

	private int length;

	/*
	 * (non-Javadoc)
	 * 
	 * @see jp.bitmeister.asn1.codec.ber.BerOctets#isConstructed()
	 */
	@Override
	boolean isConstructed() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see jp.bitmeister.asn1.codec.ber.BerOctets#octetsLength()
	 */
	@Override
	int octetsLength() {
		return length;
	}

	/**
	 * Appends an element to this octets.
	 * 
	 * @param element
	 *            The element to be added.
	 */
	void addElement(BerOctets element) {
		elements.add(element);
		length += element.totalLength();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * jp.bitmeister.asn1.codec.ber.BerOctets#writeContents(java.io.OutputStream
	 * )
	 */
	@Override
	void writeContents(OutputStream out) throws IOException {
		for (BerOctets o : elements) {
			o.write(out);
		}
	}

}
